@@ -715,7 +715,7 @@ return $_[0] !~ /^\//;
 }
 
 # list_mail_users([max], [filterfunc])
-# Returns getpw* style structures for all users who can recieve mail. Those with
+# Returns getpw* style structures for all users who can receive mail. Those with
 # duplicate info are skipped.
 sub list_mail_users
 {
@@ -1048,7 +1048,7 @@ sub get_signature
 local $sf = &get_signature_file($_[0]);
 $sf || return undef;
 local $sig;
-open(SIG, $sf) || return undef;
+open(SIG, "<".$sf) || return undef;
 while(<SIG>) {
 	$sig .= $_;
 	}
@@ -1119,33 +1119,33 @@ foreach my $folder (&list_user_folders($user)) {
 		# Remove mbox index
 		local $ifile = &user_index_file($folder->{'file'});
 		if (-r $ifile) {
-			&unlink_logged($ifile);
+			&unlink_file($ifile);
 			}
 		else {
-			&unlink_logged(glob("$ifile.{dir,pag,db}"));
+			&unlink_file(glob("$ifile.{dir,pag,db}"));
 			}
-		&unlink_logged("$ifile.ids");
+		&unlink_file("$ifile.ids");
 		}
 	elsif ($folder->{'type'} == 1) {
 		# Remove Maildir files file
-		&unlink_logged(&get_maildir_cachefile($folder->{'file'}));
+		&unlink_file(&get_maildir_cachefile($folder->{'file'}));
 		}
 	# Remove sort index
 	local $ifile = &folder_new_sort_index_file($folder);
 	if (-r $ifile) {
-		&unlink_logged($ifile);
+		&unlink_file($ifile);
 		}
 	else {
-		&unlink_logged(glob("$ifile.{dir,pag,db}"));
+		&unlink_file(glob("$ifile.{dir,pag,db}"));
 		}
 	}
 # Remove read file
 local $read = &user_read_dbm_file($user);
 if (-r $read) {
-	&unlink_logged($read);
+	&unlink_file($read);
 	}
 else {
-	&unlink_logged(glob("$read.{dir,pag,db}"));
+	&unlink_file(glob("$read.{dir,pag,db}"));
 	}
 }
 
